@@ -16,7 +16,7 @@
 
 [@@@ocaml.warning "+a-4-9-30-40-41-42"]
 
-let name_expr (named : Flambda.named) ~name : Flambda.t =
+let name_expr ~name (named : Flambda.named) : Flambda.t =
   let var =
     Variable.create
       ~current_compilation_unit:(Compilation_unit.get_current_exn ())
@@ -24,6 +24,14 @@ let name_expr (named : Flambda.named) ~name : Flambda.t =
   in
   Flambda.create_let var named (Var var)
 
+let name_expr_from_var ~var (named : Flambda.named) : Flambda.t =
+  let var =
+    Variable.rename
+      ~current_compilation_unit:(Compilation_unit.get_current_exn ())
+      var
+  in
+  Flambda.create_let var named (Var var)
+
 let find_declaration cf ({ funs } : Flambda.function_declarations) =
   Variable.Map.find (Closure_id.unwrap cf) funs
 
@@ -313,12 +321,14 @@ let toplevel_substitution sb tree =
 (* CR-someday mshinwell: Fix [Flambda_iterators] so this can be implemented
    properly. *)
 let toplevel_substitution_named sb named =
-  let expr = name_expr named ~name:"toplevel_substitution_named" in
+  let name = Internal_variable_names.toplevel_substitution_named in
+  let expr = name_expr named ~name in
   match toplevel_substitution sb expr with
   | Let let_expr -> let_expr.defining_expr
   | _ -> assert false
 
-let make_closure_declaration ~id ~body ~params ~stub : Flambda.t =
+let make_closure_declaration
+      ~is_classic_mode ~id ~body ~params ~stub : Flambda.t =
   let free_variables = Flambda.free_variables body in
   let param_set = Parameter.Set.vars params in
   if not (Variable.Set.subset param_set free_variables) then begin
@@ -339,6 +349,7 @@ let make_closure_declaration ~id ~body ~params ~stub : Flambda.t =
     Flambda.create_function_declaration ~params:(List.map subst_param params)
       ~body ~stub ~dbg:Debuginfo.none ~inline:Default_inline
       ~specialise:Default_specialise ~is_a_functor:false
+      ~closure_origin:(Closure_origin.create (Closure_id.wrap id))
   in
   assert (Variable.Set.equal (Variable.Set.map subst free_variables)
     function_declaration.free_variables);
@@ -357,12 +368,13 @@ let make_closure_declaration ~id ~body ~params ~stub : Flambda.t =
   in
   let compilation_unit = Compilation_unit.get_current_exn () in
   let set_of_closures_var =
-    Variable.create "set_of_closures"
+    Variable.create Internal_variable_names.set_of_closures
       ~current_compilation_unit:compilation_unit
   in
   let set_of_closures =
     let function_decls =
       Flambda.create_function_declarations
+        ~is_classic_mode
         ~funs:(Variable.Map.singleton id function_declaration)
     in
     Flambda.create_set_of_closures ~function_decls ~free_vars
@@ -376,7 +388,7 @@ let make_closure_declaration ~id ~body ~params ~stub : Flambda.t =
       }
   in
   let project_closure_var =
-    Variable.create "project_closure"
+    Variable.create Internal_variable_names.project_closure
       ~current_compilation_unit:compilation_unit
   in
   Flambda.create_let set_of_closures_var (Set_of_closures set_of_closures)
@@ -466,7 +478,8 @@ let make_closure_map program =
     { function_decls } ->
     Variable.Map.iter (fun var _ ->
         let closure_id = Closure_id.wrap var in
-        map := Closure_id.Map.add closure_id function_decls !map)
+        let set_of_closures_id = function_decls.set_of_closures_id in
+        map := Closure_id.Map.add closure_id set_of_closures_id !map)
       function_decls.funs
   in
   Flambda_iterators.iter_on_set_of_closures_of_program
@@ -474,16 +487,18 @@ let make_closure_map program =
     ~f:add_set_of_closures;
   !map
 
-let make_closure_map' input =
-  let map = ref Closure_id.Map.empty in
-  let add_set_of_closures _ (function_decls : Flambda.function_declarations) =
-    Variable.Map.iter (fun var _ ->
-        let closure_id = Closure_id.wrap var in
-        map := Closure_id.Map.add closure_id function_decls !map)
-      function_decls.funs
-  in
-  Set_of_closures_id.Map.iter add_set_of_closures input;
-  !map
+let all_lifted_constant_closures program =
+  List.fold_left (fun unchanged flambda ->
+      match flambda with
+      | (_, Flambda.Set_of_closures { function_decls = { funs } }) ->
+        Variable.Map.fold
+          (fun key (_ : Flambda.function_declaration) acc ->
+             Closure_id.Set.add (Closure_id.wrap key) acc)
+          funs
+          unchanged
+      | _ -> unchanged)
+    Closure_id.Set.empty
+    (all_lifted_constants program)
 
 let all_lifted_constant_sets_of_closures program =
   let set = ref Set_of_closures_id.Set.empty in
@@ -511,34 +526,6 @@ let all_sets_of_closures_map program =
           set_of_closures !r);
   !r
 
-let all_function_decls_indexed_by_set_of_closures_id program =
-  Set_of_closures_id.Map.map
-    (fun { Flambda. function_decls; _ } -> function_decls)
-    (all_sets_of_closures_map program)
-
-let all_function_decls_indexed_by_closure_id program =
-  let aux_fun function_decls fun_var _ map =
-    let closure_id = Closure_id.wrap fun_var in
-    Closure_id.Map.add closure_id function_decls map
-  in
-  let aux _ ({ function_decls; _ } : Flambda.set_of_closures) map =
-    Variable.Map.fold (aux_fun function_decls) function_decls.funs map
-  in
-  Set_of_closures_id.Map.fold aux (all_sets_of_closures_map program)
-    Closure_id.Map.empty
-
-let make_variable_symbol var =
-  Symbol.create (Compilation_unit.get_current_exn ())
-    (Linkage_name.create
-       (Variable.unique_name (Variable.rename var)))
-
-let make_variables_symbol vars =
-  let name =
-    String.concat "_and_"
-      (List.map (fun var -> Variable.unique_name (Variable.rename var)) vars)
-  in
-  Symbol.create (Compilation_unit.get_current_exn ()) (Linkage_name.create name)
-
 let substitute_read_symbol_field_for_variables
     (substitution : (Symbol.t * int list) Variable.Map.t)
     (expr : Flambda.t) =
@@ -549,8 +536,10 @@ let substitute_read_symbol_field_for_variables
       | [] -> Symbol symbol
       | [i] -> Read_symbol_field (symbol, i)
       | h :: t ->
-          let block = Variable.create "symbol_field_block" in
-          let field = Variable.create "get_symbol_field" in
+          let block_name = Internal_variable_names.symbol_field_block in
+          let block = Variable.create block_name in
+          let field_name = Internal_variable_names.get_symbol_field in
+          let field = Variable.create field_name in
           Expr (
             Flambda.create_let block (make_named t)
               (Flambda.create_let field
